<?php

namespace Intergo\Providers\Enums;

use Intergo\Providers\MessageManager\Drivers\NthViberDriver;

class FailedReasonEnum extends BaseEnum
{
    const DEFAULT_CUSTOMER_FAILED_REASON = 'Technical error 5008 - Contact Support';

    // ITSMPP ERROR RESPONSES ON THE API EG. resulted in a `400 Bad Request` response:\n{\"error\":\"no_money\"}
    const NO_ACTIVE_AGGREGATOR = "no_active_aggregator";
    const USER_NOT_ALLOWED_SENDING_TO_THIS_AGGREGATOR = "user_not_allowed_sending_to_this_aggregator";
    const NO_MONEY = "no_money";
    const UNKNOKN_ERROR = "unknown_error";

    /**
     * Get the error type so we can use it on Graylog or other places to group the rejected errors from providers
     * @param $reason
     * @return mixed|string
     */
    public static function getRejectedErrorType($reason)
    {
        $error_types = self::getConstantValues();
        foreach($error_types as $error_type){
            if(strstr($reason, $error_type) !== FALSE){
                return $error_type;
            }
        }
        return self::UNKNOKN_ERROR;
    }

    /**
     * Get the error type so we can use it on Graylog or other places to group the rejected errors from providers
     * @param $reason
     * @return mixed|string
     */
    public static function getViberFailedReason(string $viber_status_code)
    {
        switch ($viber_status_code) {
            case strstr($viber_status_code, NthViberDriver::USER_BLOCKED) !== FALSE :
                return 'Technical error 5009 - Contact Support';
            case strstr($viber_status_code, NthViberDriver::NOT_VIBER_USER) !== FALSE :
                return 'Technical error 5010 - Contact Support';
            case strstr($viber_status_code, NthViberDriver::NO_SUITABLE_DEVICE) !== FALSE :
                return 'Technical error 5011 - Contact Support';
            case strstr($viber_status_code, NthViberDriver::TIMEOUT) !== FALSE :
                return 'Technical error 5012 - Contact Support';
            case strstr($viber_status_code, NthViberDriver::INVALID_PHONE_NUMBER_FORMAT) !== FALSE :
                return 'Technical error 5013 - Contact Support';
            case strstr($viber_status_code, NthViberDriver::REQUEST_ERROR) !== FALSE :
                return 'Technical error 5014 - Contact Support';
            case strstr($viber_status_code, NthViberDriver::INTERNAL_ERROR) !== FALSE :
                return 'Technical error 5015 - Contact Support';
        }
        return null;
    }

    /**
     * Get the 'failed_reason' that will be displayed to the customer.
     *
     * Here we map the error messages/statuses from providers with the error messages that will be visible to customers.
     *
     * @see https://app.asana.com/0/754800746265705/1201445985948718/f
     * @see https://intergo.atlassian.net/wiki/spaces/SMSD/pages/1660387332/User+Error+Codes
     * @param string $providerReason
     * @return string
     */
    public static function getCustomerFailedReason(string $providerReason)
    {

        $internal_reason = self::getViberFailedReason($providerReason);

        if ($internal_reason ) {
            return  $internal_reason;
        }

        switch ($providerReason) {
            // Routee specific: https://docs.routee.net/docs/statuses-callbacks
            case 'Undelivered on Handset':
                return 'Technical error 5001 - Undelivered on Handset';
            case 'Unknown Number':
                return 'Technical error 5002 - Unknown Number';
            case 'Roaming':
                return 'Technical error 5003 - Roaming';
            case 'Call Barred':
                return 'Technical error 5004 - Call Barred';
            case 'Unknown Status':
                return 'Technical error 5005 - Unknown Status';
            case 'Carrier Violation':
                return 'Technical error 5006 - Carrier Violation';
            case 'Expired':
                return 'Technical error 5007 - Expired';
            // If the error message/status from provider is not recognized/mapped:
            default:
                return self::DEFAULT_CUSTOMER_FAILED_REASON;
        }
    }
}
